@@ -1,29 +1,33 @@
 from __future__ import division
 
-import six
-
-from six.moves import xrange
-
 import hashlib
 import random
-import types
+from io import BytesIO
 import os
+import time
 from datetime import datetime
 
 from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 from django.core.files.storage import FileSystemStorage
+from django.contrib.contenttypes.fields import GenericForeignKey
 from django.contrib.contenttypes.models import ContentType
-from django.contrib.contenttypes import generic
-from django.db import models
+from django.db import connection, models
+from django.utils import six
+from django.utils.encoding import python_2_unicode_compatible
+from django.utils.six.moves import xrange
+from django.core.files.storage import default_storage, FileSystemStorage
 
 import PIL.Image
 
 from generic_plus.utils import get_relative_media_url
 
 from .exceptions import CropDusterResizeException
-from .fields import CropDusterField, ReverseForeignRelation, CropDusterImageField
+from .fields import (
+    CropDusterField, ReverseForeignRelation, CropDusterImageField,
+    CropDusterSimpleImageField)
 from .files import VirtualFieldFile
-from .resizing import Size, Box, Crop
+from .resizing import Size, Box, Crop, SizeAlias
+from .utils import process_image
 from . import settings as cropduster_settings
 
 
@@ -40,6 +44,7 @@ def safe_str_path(file_path):
     return file_path
 
 
+@python_2_unicode_compatible
 class Thumb(models.Model):
 
     name = models.CharField(max_length=255, db_index=True)
@@ -49,7 +54,7 @@ class Thumb(models.Model):
     # For a given thumbnail, it either has crop data or it references
     # another thumbnail with crop data
     reference_thumb = models.ForeignKey('Thumb', blank=True, null=True,
-            related_name='auto_set')
+            related_name='auto_set', on_delete=models.CASCADE)
 
     crop_x = models.PositiveIntegerField(blank=True, null=True)
     crop_y = models.PositiveIntegerField(blank=True, null=True)
@@ -58,48 +63,60 @@ class Thumb(models.Model):
 
     date_modified = models.DateTimeField(auto_now=True)
 
-    image = models.ForeignKey('Image', related_name='+', null=True, blank=True)
+    image = models.ForeignKey('Image', related_name='+', null=True, blank=True,
+        on_delete=models.CASCADE)
 
     class Meta:
         app_label = cropduster_settings.CROPDUSTER_APP_LABEL
         db_table = '%s_thumb' % cropduster_settings.CROPDUSTER_DB_PREFIX
 
-    def __unicode__(self):
+    def __str__(self):
         return self.name
 
     @property
     def image_file(self):
-        return Image.get_file_for_size(image=self.image, size_name=self.name)
+        return Image.get_file_for_size(
+            image=self.image, size_name=self.name,
+            tmp=not(getattr(self.image, 'pk', None)))
 
     @property
     def url(self):
         return self.image_file.url if self.image_file else ''
 
+    @property
+    def cache_safe_url(self):
+        """A URL that includes a GET parameter that changes upon modification"""
+        cache_buster = time.mktime(self.date_modified.timetuple())
+        return "%s?mod=%d" % (self.url, cache_buster)
+
     @property
     def path(self):
         return self.image_file.path if self.image_file else ''
 
+    @property
+    def image_name(self):
+        return self.image_file.name if self.image_file else ''
+
     def save(self, *args, **kwargs):
-        if self.pk:
+        if self.pk and self.image_id:
             try:
-                orig_thumb = Thumb.objects.select_for_update().get(pk=self.pk)
-            except Thumb.DoesNotExist:
+                # save new file without tmp suffix
+                tmp_image_path = self.image.get_image_path(self.name, tmp=True)
+                image_path = self.image.get_image_path(self.name)
+                with default_storage.open(tmp_image_path) as tmp_file:
+                    with default_storage.open(image_path, 'wb') as f:
+                        f.write(tmp_file.read())
+                # delete tmp file
+                default_storage.delete(tmp_image_path)
+            except (IOError, OSError):
                 pass
-            else:
-                if self.image_id and not orig_thumb.image_id:                    
-                    try:
-                        os.rename(
-                            self.image.get_image_path(self.name, tmp=True),
-                            self.image.get_image_path(self.name))
-                    except (IOError, OSError):
-                        pass
         return super(Thumb, self).save(*args, **kwargs)
 
     def to_dict(self):
         """Returns a dict of the thumb's values which are JSON serializable."""
         dct = {}
         for k, v in six.iteritems(vars(self)):
-            if isinstance(v, (six.string_types, float, bool, types.NoneType)):
+            if isinstance(v, (six.string_types, float, bool, type(None))):
                 dct[k] = v
             if isinstance(v, six.integer_types):
                 dct[k] = v
@@ -116,7 +133,7 @@ class Thumb(models.Model):
             return None
         return Box(x1, y1, x2, y2)
 
-    def crop(self, output_filename, original_image=None, w=None, h=None, min_w=None, min_h=None, max_w=None, max_h=None):
+    def crop(self, original_image=None, size=None, w=None, h=None):
         if original_image is None:
             if not self.pk:
                 raise Exception(
@@ -135,43 +152,46 @@ class Thumb(models.Model):
             raise Exception("Cannot crop thumbnail without crop data")
         crop = Crop(crop_box, original_image)
 
-        self.width = w or None
-        self.height = h or None
+        width = size.w or w
+        height = size.h or h
 
         if self.reference_thumb:
             best_fit_kwargs = {
-                'min_w': min_w or self.width,
-                'min_h': min_h or self.height,
+                'min_w': size.min_w or width,
+                'min_h': size.min_h or height,
             }
-            if self.width and self.height:
-                best_fit_kwargs.update({'w': self.width, 'h': self.height})
-            fit = crop.best_fit(**best_fit_kwargs)
-            if not self.width and not self.height:
-                self.width, self.height = fit.box.size
-            elif not self.width:
-                width = fit.box.w * (self.height / fit.box.h)
-                self.width = min(int(round(width)), crop.bounds.w)
-            elif not self.height:
-                height = fit.box.h * (self.width / fit.box.w)
-                self.height = min(int(round(height)), crop.bounds.h)
-            new_image = fit.create_image(output_filename, width=self.width, height=self.height, max_w=max_w, max_h=max_h)
-        else:
-            if w and h:
-                self.width = w
-                self.height = h
-            elif w:
-                height = crop_box.h * (w / crop_box.w)
-                self.height = min(int(round(height)), crop.bounds.h)
-            elif h:
-                width = crop_box.w * (h / crop_box.h)
-                self.width = min(int(round(width)), crop.bounds.w)
-            else:
-                self.width, self.height = crop.box.size
+            if width and height:
+                best_fit_kwargs.update({'w': width, 'h': height})
+            crop = crop.best_fit(**best_fit_kwargs)
+        if not width and not height:
+            width, height = crop.box.size
+        elif not width:
+            width = crop.box.w * (height / crop.box.h)
+            width = min(int(round(width)), crop.bounds.w)
+        elif not height:
+            height = crop.box.h * (width / crop.box.w)
+            height = min(int(round(height)), crop.bounds.h)
 
-            new_image = crop.create_image(output_filename, width=self.width, height=self.height, max_w=max_w, max_h=max_h)
+        new_w, new_h = crop.box.size
+        if new_w < width or new_h < height:
+            raise CropDusterResizeException(
+                u"Crop box (%dx%d) is too small for resize to (%dx%d)" % (new_w, new_h, width, height))
 
-        self.width, self.height = new_image.size
-        return new_image
+        # Scale our initial width and height based on the max_w and max_h
+        max_scales = []
+        if size.max_w and size.max_w < width:
+            max_scales.append(size.max_w / width)
+        if size.max_h and size.max_h < height:
+            max_scales.append(size.max_h / height)
+        if max_scales:
+            max_scale = min(max_scales)
+            width = int(round(width * max_scale))
+            height = int(round(height * max_scale))
+
+        self.width = width
+        self.height = height
+
+        return crop
 
 
 class StrFileSystemStorage(FileSystemStorage):
@@ -189,25 +209,27 @@ class StrFileSystemStorage(FileSystemStorage):
 image_storage = StrFileSystemStorage()
 
 
+def generate_filename(instance, filename):
+    return filename
+
+
+@python_2_unicode_compatible
 class Image(models.Model):
 
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
     object_id = models.PositiveIntegerField(null=True, blank=True)
-    content_object = generic.GenericForeignKey('content_type', 'object_id')
-    field_identifier = models.SlugField(null=True, blank=True)
+    content_object = GenericForeignKey('content_type', 'object_id')
+    field_identifier = models.SlugField(null=False, blank=True, default="")
 
     prev_object_id = models.PositiveIntegerField(null=True, blank=True)
-    prev_content_object = generic.GenericForeignKey('content_type', 'prev_object_id')
-
-    @staticmethod
-    def generate_filename(instance, filename):
-        return filename
+    prev_content_object = GenericForeignKey('content_type', 'prev_object_id')
 
     width = models.PositiveIntegerField(blank=True, null=True)
     height = models.PositiveIntegerField(blank=True, null=True)
 
-    image = models.ImageField(db_index=True, upload_to=generate_filename, db_column='path',
-        storage=image_storage, width_field='width', height_field='height')
+    image = CropDusterSimpleImageField(db_index=True,
+        upload_to=generate_filename, db_column='path',
+        width_field='width', height_field='height')
 
     thumbs = ReverseForeignRelation(Thumb, field_name='image')
 
@@ -217,17 +239,23 @@ class Image(models.Model):
     attribution = models.CharField(max_length=255, blank=True, null=True)
     attribution_link = models.URLField(max_length=255, blank=True, null=True)
     caption = models.TextField(blank=True, null=True)
+    alt_text = models.TextField("Alt Text", blank=True, default="")
 
     class Meta:
         app_label = cropduster_settings.CROPDUSTER_APP_LABEL
         db_table = '%s_image' % cropduster_settings.CROPDUSTER_DB_PREFIX
         unique_together = ("content_type", "object_id", "field_identifier")
 
-    def __unicode__(self):
+    def __str__(self):
         return self.get_image_url()
 
+    # TODO: deprecated
     @property
     def path(self):
+        return self.name
+
+    @property
+    def name(self):
         return self.image.name if self.image else None
 
     @property
@@ -247,7 +275,7 @@ class Image(models.Model):
             image = VirtualFieldFile(image)
         if not image:
             return None
-        path, basename = os.path.split(safe_str_path(image.path))
+        path, basename = os.path.split(safe_str_path(image.name))
         filename, extension = os.path.splitext(basename)
         if size_name == 'preview':
             size_name = '_preview'
@@ -255,29 +283,34 @@ class Image(models.Model):
             size_name = '%s_tmp' % size_name
         return VirtualFieldFile(
             '/'.join([
-                get_relative_media_url(path),
+                path,
                 safe_str_path(size_name) + extension]))
 
     @classmethod
     def save_preview_file(cls, image_file, preview_w=None, preview_h=None):
-        pil_img = PIL.Image.open(safe_str_path(image_file.path))
+        with image_file as f:
+            f.open()
+            pil_img = PIL.Image.open(BytesIO(f.read()))
+            pil_img.filename = f.name
         orig_w, orig_h = pil_img.size
 
         preview_w = preview_w or cropduster_settings.CROPDUSTER_PREVIEW_WIDTH
         preview_h = preview_h or cropduster_settings.CROPDUSTER_PREVIEW_HEIGHT
 
         resize_ratio = min(preview_w / orig_w, preview_h / orig_h)
-        if resize_ratio < 1:
-            w = int(round(orig_w * resize_ratio))
-            h = int(round(orig_h * resize_ratio))
-            preview_img = pil_img.resize((w, h), PIL.Image.ANTIALIAS)
-        else:
-            preview_img = pil_img
+
+        def fit_preview(im):
+            if resize_ratio < 1:
+                w = int(round(orig_w * resize_ratio))
+                h = int(round(orig_h * resize_ratio))
+                preview_img = im.resize((w, h), PIL.Image.ANTIALIAS)
+            else:
+                w, h = orig_w, orig_h
+                preview_img = im
+            return preview_img
+
         preview_file = cls.get_file_for_size(image_file, '_preview')
-        img_save_params = {}
-        if preview_img.format == 'JPEG':
-            img_save_params['quality'] = cropduster_settings.get_jpeg_quality(w, h)
-        preview_img.save(safe_str_path(preview_file.path), **img_save_params)
+        process_image(pil_img, safe_str_path(preview_file.name), fit_preview)
         return preview_file
 
     def save_preview(self, preview_w=None, preview_h=None):
@@ -309,12 +342,12 @@ class Image(models.Model):
         if not converted:
             return u''
         else:
-            return converted.path
+            return converted.name
 
     def save(self, **kwargs):
         self.date_modified = datetime.now()
-        if self.field_identifier == "":
-            self.field_identifier = None
+        if self.field_identifier is None:
+            self.field_identifier = ""
         if not self.pk and self.content_type and self.object_id:
             try:
                 original = Image.objects.get(content_type=self.content_type,
@@ -333,11 +366,19 @@ class Image(models.Model):
         # If the Image has changed, we need to make sure the related field on the
         # model class has also been updated
         model_class = self.content_type.model_class()
-        for field, _ in model_class._meta.get_fields_with_model():
+
+        fields_with_models = [
+            (f, f.model if f.model != model_class else None)
+            for f in model_class._meta.get_fields()
+            if not f.is_relation
+            or f.one_to_one
+            or (f.many_to_one and f.related_model)]
+
+        for field, field_model_class in fields_with_models:
+            field_model_class = field_model_class or model_class
             if (isinstance(field, CropDusterImageField) and
                     field.generic_field.field_identifier == self.field_identifier):
-                model_class.objects.filter(pk=self.object_id).update(**{field.attname: self.path})
-                break
+                field_model_class.objects.filter(pk=self.object_id).update(**{field.attname: self.path or ''})
 
     def get_image_url(self, size_name='original', tmp=False):
         converted = Image.get_file_for_size(self.image, size_name, tmp=tmp)
@@ -358,7 +399,7 @@ class Image(models.Model):
                 return (thumb.width, thumb.height)
 
         # Get the original size
-        if not self.image or not os.path.exists(safe_str_path(self.image.path)):
+        if not self.image or not default_storage.exists(safe_str_path(self.image.path)):
             return (0, 0)
         elif self.width and self.height:
             return (self.width, self.height)
@@ -394,26 +435,40 @@ class Image(models.Model):
             setattr(obj, cropduster_field.name, None)
             obj.save()
 
-    def save_size(self, size, thumb=None, image=None, tmp=False, standalone=False, permissive=False):
+    def save_size(self, size, thumb=None, image=None, tmp=False, standalone=False,
+                  permissive=False, skip_existing=False):
         thumbs = {}
         if not image and not self.image:
             raise Exception("Cannot save sizes without an image")
 
-        image = image or PIL.Image.open(safe_str_path(self.image.path))
+        if not image:
+            with default_storage.open(self.image.name) as f:
+                image = PIL.Image.open(f)
+                image.filename = self.image.name
 
         if standalone:
             if not StandaloneImage:
                 raise ImproperlyConfigured(u"standalone mode used, but not installed.")
-            return self._save_thumb(size, image, thumb, standalone=True)
+            return self._save_standalone_thumb(size, image, thumb)
 
         for sz in Size.flatten([size]):
+            if self.pk and skip_existing and default_storage.exists(self.get_image_path(sz.name)):
+                try:
+                    existing_thumb = self.thumbs.get(name=sz.name)
+                except Thumb.DoesNotExist:
+                    pass
+                else:
+                    thumbs[sz.name] = existing_thumb
+                    continue
             try:
-                if sz.is_auto:
+                if thumb and sz.is_auto:
                     new_thumb = self._save_thumb(sz, image, ref_thumb=thumb, tmp=tmp)
                 else:
                     thumb = new_thumb = self._save_thumb(sz, image, thumb, tmp=tmp)
             except CropDusterResizeException:
-                if permissive:
+                if permissive or not sz.required:
+                    if not sz.is_auto:
+                        thumb = new_thumb = None
                     continue
                 else:
                     raise
@@ -422,66 +477,52 @@ class Image(models.Model):
                 thumbs[sz.name] = new_thumb
         return thumbs
 
-    def _save_thumb(self, size, image=None, thumb=None, ref_thumb=None, tmp=False, standalone=False):
-        img_save_params = {}
-        if image.format == 'JPEG':
-            img_save_params['quality'] = cropduster_settings.get_jpeg_quality(self.width, self.height)
-
+    def _save_standalone_thumb(self, size, image=None, thumb=None):
         if not thumb:
-            if standalone:
-                thumb = Thumb(
-                    width=self.width, height=self.height,
-                    crop_x=0, crop_y=0, crop_w=self.width, crop_h=self.height)
-            elif self.pk:
-                try:
-                    thumb = self.thumbs.get(name=size.name)
-                except Thumb.DoesNotExist:
-                    pass
+            thumb = Thumb(
+                width=self.width, height=self.height,
+                crop_x=0, crop_y=0, crop_w=self.width, crop_h=self.height)
+        thumb.name = ''.join([random.choice('abcdefghijklmnopqrstuvwxyz0123456789') for i in xrange(0, 8)])
+        thumb_path = self.get_image_path(thumb.name)
+        thumb_crop = thumb.crop(image, size,
+            w=(size.w or thumb.crop_w),
+            h=(size.h or thumb.crop_h))
+        thumb_image = thumb_crop.create_image(thumb_path, width=thumb.width, height=thumb.height)
+        thumb_image.crop.add_xmp_to_crop(thumb_path, size, original_image=image)
+        md5 = hashlib.md5()
+        with default_storage.open(thumb_path, mode='rb') as f:
+            image_contents = f.read()
+        md5.update(image_contents)
+        thumb.name = md5.hexdigest()[0:9]
+        new_path = self.get_image_path(thumb.name)
+        with default_storage.open(new_path, 'wb') as f:
+            f.write(image_contents)
+        default_storage.delete(thumb_path)
+        return thumb
 
-        if standalone:
-            thumb.name = ''.join([random.choice('abcdefghijklmnopqrstuvwxyz0123456789') for i in xrange(0, 8)])
-        elif not thumb:
+    def _save_thumb(self, size, image=None, thumb=None, ref_thumb=None, tmp=False, commit=True):
+        image = image or PIL.Image.open(safe_str_path(self.image.path))
+        if not thumb and self.pk:
+            try:
+                thumb = self.thumbs.get(name=size.name)
+            except Thumb.DoesNotExist:
+                pass
+        if not thumb:
             thumb = Thumb(name=size.name)
         elif not thumb.name:
             thumb.name = size.name
 
-        crop_box = None
-
         if size.is_auto:
-            ref_thumb = ref_thumb or thumb.reference_thumb
-            if ref_thumb:
-                crop_box = ref_thumb.get_crop_box()
-        elif thumb:
-            crop_box = thumb.get_crop_box()
+            thumb.reference_thumb = ref_thumb or thumb.reference_thumb
 
-        if not crop_box:
-            return None
-
-        thumb.reference_thumb = ref_thumb
-
-        crop_kwargs = dict([(k, getattr(size, k))
-                            for k in ['w', 'h', 'min_w', 'min_h', 'max_w', 'max_h']])
-        if standalone and not(size.w or size.h) and (thumb.crop_w and thumb.crop_h):
-            crop_kwargs['w'] = thumb.crop_w
-            crop_kwargs['h'] = thumb.crop_h
-
-        if standalone:
-            thumb_path = self.get_image_path(thumb.name)
-        else:
-            thumb_path = self.get_image_path(size.name, tmp=tmp)
-
-        thumb_image = thumb.crop(thumb_path, image, **crop_kwargs)
+        thumb_crop = thumb.crop(image, size)
+        thumb_path = self.get_image_path(size.name, tmp=tmp)
+        thumb_image = thumb_crop.create_image(thumb_path, width=thumb.width, height=thumb.height)
 
         if StandaloneImage:
-            thumb_image.crop.add_xmp_to_crop(thumb_path, size)
+            thumb_image.crop.add_xmp_to_crop(thumb_path, size, original_image=image)
 
-        if standalone:
-            md5 = hashlib.md5()
-            with open(thumb_path, mode='rb') as f:
-                md5.update(f.read())
-            thumb.name = md5.hexdigest()[0:9]
-            os.rename(thumb_path, self.get_image_path(thumb.name))
-        else:
+        if commit:
             thumb.save()
         return thumb
 
@@ -497,40 +538,3 @@ except:
     @six.add_metaclass(FalseMeta)
     class StandaloneImage(object):
         DoesNotExist = type('DoesNotExist', (ObjectDoesNotExist,), {})
-
-
-try:
-    from south.modelsinspector import add_introspection_rules, add_ignored_fields
-except ImportError:
-    pass
-else:
-    add_ignored_fields(["^cropduster\.fields\.ReverseForeignRelation"])
-
-    def converter(value):
-        """Custom south converter so that Size objects serialize properly"""
-        if isinstance(value, Size):
-            return value.__serialize__()
-        try:
-            is_sizes_list = all([isinstance(sz, Size) for sz in value])
-        except TypeError:
-            pass
-        else:
-            if is_sizes_list:
-                return [sz.__serialize__() for sz in value]
-        return repr(value)
-
-
-    add_introspection_rules(rules=[
-        (
-            (CropDusterField,),
-            [],
-            {
-                "to": ["rel.to", {}],
-                "symmetrical": ["rel.symmetrical", {"default": True}],
-                "object_id_field": ["object_id_field_name", {"default": "object_id"}],
-                "content_type_field": ["content_type_field_name", {"default": "content_type"}],
-                "blank": ["blank", {"default": True}],
-                "sizes": ["sizes", {"converter": converter}],
-            },
-        ),
-    ], patterns=["^cropduster\.fields\.CropDusterField"])
