@@ -380,6 +380,10 @@ CKEDITOR.dialog.add('cropduster', function (editor) {
             cropdusterIframe = {
                 setup: function (domId, baseUrl) {
                     this.iframeElement = CKEDITOR.document.getById(domId).getChild(0)
+                    this.iframeElement.$.src = 'about:blank';
+                    try {
+                        this.iframeElement.$.contentDocument.body.innerHTML = "";
+                    } catch(e) {}
                     this.baseUrl = baseUrl;
                     this.callback_fn = domId.replace(/_uiElement$/, '') + '_callback';
 
@@ -474,6 +478,18 @@ CKEDITOR.dialog.add('cropduster', function (editor) {
         }
     }
 
+    tabElements.push({
+        id: 'alt',
+        type: 'text',
+        label: lang.alt,
+        setup: function (widget) {
+            this.setValue(widget.data.alt);
+        },
+        commit: function (widget) {
+            widget.setData('alt', this.getValue());
+        }
+    });
+
     return {
         title: lang.title,
         minWidth: 650,
@@ -545,16 +561,6 @@ CKEDITOR.dialog.add('cropduster', function (editor) {
                 validate: function() {
                     return true;
                 }
-            }, {
-                id: 'alt',
-                type: 'text',
-                label: lang.alt,
-                setup: function (widget) {
-                    this.setValue(widget.data.alt);
-                },
-                commit: function (widget) {
-                    widget.setData('alt', this.getValue());
-                }
             }, {
                 type: 'hbox',
                 widths: ['25%', '25%', '50%'],
@@ -601,4 +607,4 @@ CKEDITOR.dialog.add('cropduster', function (editor) {
             }]
         }]
     };
-});
\ No newline at end of file
+});
