@@ -1,10 +1,10 @@
 from functools import update_wrapper
 
-import six
+from logging import getLogger
 
 from django import http
-from django.utils.log import getLogger
 from django.utils.decorators import classonlymethod
+from django.utils import six
 
 
 logger = getLogger('django.request')
